@@ -4,14 +4,16 @@ import numpy as np
 from scipy import interpolate
 
 from bmlab.models.regions import regions_merge_add_region
+from bmlab.serializer import Serializer
 
 logger = logging.getLogger(__name__)
 
 
-class CalibrationModel(object):
+class CalibrationModel(Serializer):
 
     def __init__(self):
         self.calib_times = {}
+        self.spectra = {}
         self.brillouin_regions = {}
         self.rayleigh_regions = {}
         self.brillouin_fits = BrillouinFitSet()
@@ -20,6 +22,13 @@ class CalibrationModel(object):
         self.vipa_params = {}
         self.frequencies = {}
 
+        self.frequency_by_calib_key_interpolators = {}
+        self.frequencies_by_time_interpolator = None
+        self.frequency_by_time_interpolator = None
+
+    def post_deserialize(self):
+        self.refresh_frequency_interpolators()
+
     def add_brillouin_region(self, calib_key, region):
         if calib_key not in self.brillouin_regions:
             self.brillouin_regions[calib_key] = []
@@ -96,22 +105,37 @@ class CalibrationModel(object):
     def clear_rayleigh_fits(self, calib_key):
         self.rayleigh_fits.clear(calib_key)
 
+    def set_spectra(self, calib_key, spectra):
+        self.spectra[calib_key] = spectra
+
+    def get_spectra(self, calib_key):
+        spectra = self.spectra.get(calib_key)
+        if spectra:
+            return spectra
+        return None
+
     def get_sorted_peaks(self, calib_key, frame_num):
         """
         Returns the sorted centers of all fitted peaks
         of a given calibration and frame.
-        :param calib_key: The calibration key
-        :param frame_num: The frame number
-        :return: sorted np.array of all peaks
+
+        Parameters
+        ----------
+        calib_key: The calibration key
+        frame_num: The frame number
+
+        Returns
+        -------
+        sorted np.array of all peaks
         """
         peaks = []
         # Search all fits for given calib_key and frame_num
         for key, fit in self.rayleigh_fits.fits.items():
-            if (fit.calib_key == calib_key) & (fit.frame_num == frame_num):
+            if (fit.calib_key == calib_key) and (fit.frame_num == frame_num):
                 peaks.append(fit.w0)
 
         for key, fit in self.brillouin_fits.fits.items():
-            if (fit.calib_key == calib_key) & (fit.frame_num == frame_num):
+            if (fit.calib_key == calib_key) and (fit.frame_num == frame_num):
                 for w0 in fit.w0s:
                     peaks.append(w0)
 
@@ -123,67 +147,55 @@ class CalibrationModel(object):
     def set_frequencies(self, calib_key, time, frequencies):
         self.frequencies[calib_key] = frequencies
         self.calib_times[calib_key] = time
+        self.refresh_frequency_interpolators()
 
     def clear_frequencies(self, calib_key):
-        del self.frequencies[calib_key]
-        del self.calib_times[calib_key]
-
-    def get_frequencies_by_calib_key(self, calib_key):
-        """
-        Returns the complete frequency axis for a given
-        calibration
-        :param calib_key: The key of the calibration
-        :return: The frequency axis in Hz
-        """
         if calib_key in self.frequencies:
-            return self.frequencies[calib_key]
+            del self.frequencies[calib_key]
+        if calib_key in self.calib_times:
+            del self.calib_times[calib_key]
+        self.refresh_frequency_interpolators()
 
-    def get_frequency_by_calib_key(self, position, calib_key):
+    def refresh_frequency_interpolators(self):
         """
-        Returns the frequency of a peak position on the
-        spectrum for a given calibration
-        :param position: The position(s) of the peak(s)
-        on the spectrum
-        :param calib_key: The key of the calibration
-        :return: The corresponding frequency in Hz
-        """
-        # TODO Move the interpolation out of this function
-        #  (only needs to be done once for a calibration key)
-        #  (not so critical at the moment, since it's only done
-        #  for calibrating)
-        frequencies = self.get_frequencies_by_calib_key(calib_key)
-        if not frequencies:
-            return
-        frequency = np.mean(np.array(frequencies), axis=0)
+        This function creates the interpolator to get
+        the frequencies of points in a spectrum
 
-        xdata = np.arange(len(frequency))
-        f = interpolate.interp1d(xdata, frequency)
-        return f(position)
-
-    def get_frequencies_by_time(self, time):
-        """
-        Returns the complete frequency axis for a given
-        time
-        :param time: The time
-        :return: The frequency axis in Hz
+        Returns
+        -------
         """
+        # Reset all interpolators
+        self.frequency_by_calib_key_interpolators = {}
+        self.frequencies_by_time_interpolator = None
+        self.frequency_by_time_interpolator = None
 
-        # Sort calibration keys by time
         sorted_keys = sorted(self.calib_times,
                              key=self.calib_times.get)
-        if not sorted_keys:
-            return None
+        # Don't do anything if there are not calibrations
+        if len(sorted_keys) < 1:
+            return
 
-        # If we only have one time point, simply return
-        #  the complete frequency axis
+        """
+        Create the interpolator for getting a frequency in a
+        calibration spectrum
+        """
+        for calib_key in self.frequencies:
+            frequencies = self.get_frequencies_by_calib_key(calib_key)
+            if frequencies is None:
+                return
+            frequency = np.mean(np.array(frequencies), axis=0)
+
+            xdata = np.arange(len(frequency))
+            self.frequency_by_calib_key_interpolators[calib_key] =\
+                interpolate.interp1d(xdata, frequency)
+
+        """
+        Create the interpolator for getting frequencies by time
+        """
         if len(sorted_keys) < 2:
-            return np.nanmean(self.frequencies[sorted_keys[0]], 0)
+            self.frequencies_by_time_interpolator = \
+                lambda time: np.nanmean(self.frequencies[sorted_keys[0]], 0)
         else:
-            # TODO Move the interpolation out of this function
-            #  (only needs to be done once)
-            #  (not so critical at the moment, since this
-            #  function is only used for showing the frequency
-            #  axis in the peak-selection panel)
             calib_times_array = []
             frequencies = []
             for key in sorted_keys:
@@ -193,43 +205,12 @@ class CalibrationModel(object):
             calib_times_array = np.array(calib_times_array)
             frequencies = np.squeeze(frequencies)
 
-            f = interpolate.interp1d(calib_times_array, frequencies, axis=0)
-            return f(time)
+            self.frequencies_by_time_interpolator =\
+                interpolate.interp1d(calib_times_array, frequencies, axis=0)
 
-    def get_frequency_by_time(self, time, position):
         """
-        Returns the frequency of a peak position on the
-        spectrum for a given time
-        :param time: The time
-        :param position: The position(s) of the peak(s)
-        on the spectrum
-        :return: The corresponding frequency in Hz
+        Create the interpolator for getting a frequency by position and time
         """
-        # Cannot execute if ndims are unequal
-        if np.ndim(time) is not np.ndim(position):
-            return None
-
-        shape_time = np.shape(time)
-        shape_position = np.shape(position)
-
-        # In case the arrays don't have the same shape,
-        # we repeat the time array
-        if shape_time is not shape_position:
-            time = np.tile(
-                time,
-                (np.array(shape_position) - np.array(shape_time)) + 1
-            )
-
-        # TODO Move the interpolation out of this function
-        #  (only needs to be done once)
-        #  (!!! quite critical because this function is called
-        #  for every measurement point)
-        # Sort calibration keys by time
-        sorted_keys = sorted(self.calib_times,
-                             key=self.calib_times.get)
-        if not sorted_keys:
-            return None
-
         calib_times_array = []
         frequencies = []
         for key in sorted_keys:
@@ -246,7 +227,8 @@ class CalibrationModel(object):
         if len(sorted_keys) < 2:
             frequencies = np.squeeze(frequencies)
             f = interpolate.interp1d(indices, frequencies)
-            return f(position)
+            self.frequency_by_time_interpolator =\
+                lambda time, position: f(position)
         # Otherwise we can interpolate by time as well
         elif len(calib_times_array) < 3:
             f = interpolate.RegularGridInterpolator(
@@ -255,7 +237,8 @@ class CalibrationModel(object):
                 method='linear',
                 bounds_error=False
             )
-            return f((time, position))
+            self.frequency_by_time_interpolator = \
+                lambda time, position: f((time, position))
         else:
             # If we only have three entries we cannot use a
             # third degree spline
@@ -266,10 +249,92 @@ class CalibrationModel(object):
                 frequencies,
                 kx=degree
             )
-            return f(time, position, grid=False)
+            self.frequency_by_time_interpolator = \
+                lambda time, position: f(time, position, grid=False)
+
+    def get_frequencies_by_calib_key(self, calib_key):
+        """
+        Returns the complete frequency axis for a given
+
+        Parameters
+        ----------
+        calib_key: The key of the calibration
+
+        Returns
+        -------
+        The frequency axis in Hz
+        """
+        if calib_key in self.frequencies:
+            return self.frequencies[calib_key]
+
+    def get_frequency_by_calib_key(self, position, calib_key):
+        """
+        Returns the frequency of a peak position on the
+        spectrum for a given calibration
+
+        Parameters
+        ----------
+        position: The position(s) of the peak(s)
+        calib_key: The key of the calibration
+
+        Returns
+        -------
+        The corresponding frequency in Hz
+        """
+        if calib_key in self.frequency_by_calib_key_interpolators:
+            return self.frequency_by_calib_key_interpolators[
+                calib_key](position)
+
+    def get_frequencies_by_time(self, time):
+        """
+        Returns the complete frequency axis for a given
+
+        Parameters
+        ----------
+        time: The time
+
+        Returns
+        -------
+        The frequency axis in Hz
+        """
+        if self.frequencies_by_time_interpolator is not None:
+            return self.frequencies_by_time_interpolator(time)
+
+    def get_frequency_by_time(self, time, position):
+        """
+        Returns the frequency of a peak position on the
+        spectrum for a given time
+
+        Parameters
+        ----------
+        time: The time
+        position: The position(s) of the peak(s)
+        on the spectrum
+
+        Returns
+        -------
+        The corresponding frequency in Hz
+        """
+        # Cannot execute if ndims are unequal
+        if np.ndim(time) is not np.ndim(position):
+            return None
+
+        shape_time = np.shape(time)
+        shape_position = np.shape(position)
+
+        # In case the arrays don't have the same shape,
+        # we repeat the time array
+        if shape_time is not shape_position:
+            time = np.tile(
+                time,
+                (np.array(shape_position) - np.array(shape_time)) + 1
+            )
+
+        if self.frequency_by_time_interpolator is not None:
+            return self.frequency_by_time_interpolator(time, position)
 
 
-class FitSet(object):
+class FitSet(Serializer):
 
     def __init__(self):
         self.fits = {}
@@ -300,7 +365,7 @@ class FitSet(object):
             del self.fits[key]
 
 
-class RayleighFitSet(FitSet):
+class RayleighFitSet(FitSet, Serializer):
 
     def average_fits(self, calib_key, region_key):
         w0s = []
@@ -314,7 +379,7 @@ class RayleighFitSet(FitSet):
         return None
 
 
-class BrillouinFitSet(FitSet):
+class BrillouinFitSet(FitSet, Serializer):
 
     def average_fits(self, calib_key, region_key):
         w0s = []
@@ -329,7 +394,7 @@ class BrillouinFitSet(FitSet):
         return None
 
 
-class RayleighFit(object):
+class RayleighFit(Serializer):
 
     def __init__(self, calib_key, region_key, frame_num,
                  w0, fwhm, intensity, offset):
@@ -342,7 +407,7 @@ class RayleighFit(object):
         self.offset = offset
 
 
-class BrillouinFit(object):
+class BrillouinFit(Serializer):
 
     def __init__(self, calib_key, region_key, frame_num,
                  w0s, fwhms, intensities, offset):
